import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.qpid.client.AMQConnection;

/**
 * Message producer example, sends message to a queue.
 */
public class FailoverProducer
{
    private static final String CLASS = "FailoverProducer";

    private int numMessages = 5000;
    private short deliveryMode = DeliveryMode.NON_PERSISTENT;

    public FailoverProducer ()
    {
        deliveryMode = (short)(Boolean.getBoolean("persistent")? 
                DeliveryMode.PERSISTENT:DeliveryMode.NON_PERSISTENT);
        
        numMessages = Integer.getInteger("msg_count", 5000);
    }

    public static void main(String[] args)
    {        
        FailoverProducer producer = new FailoverProducer();
        producer.runTest();
    }
    
    private void runTest()
    {
       
        try
        {
            Properties properties = new Properties();
            properties.load(this.getClass().getResourceAsStream("failover.properties"));

            Context ctx = new InitialContext(properties);

            Destination destination = Boolean.getBoolean("persistent")? 
                    (Destination)ctx.lookup("directDuraQueue"):(Destination)ctx.lookup("directQueue");

            ConnectionFactory conFac = (ConnectionFactory)ctx.lookup("qpidConnectionfactory");
            Connection connection = conFac.createConnection();
            connection.start();

            System.out.println(((AMQConnection)connection).getFailoverPolicy());
            
            connection.setExceptionListener(new ExceptionListener()
            {
                public void onException(JMSException e)
                {
                    e.printStackTrace();
                }
            });

            System.out.println(CLASS + ": Creating a non-transacted, auto-acknowledged session");
            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

            System.out.println(CLASS + ": Creating a Message Producer");
            MessageProducer messageProducer = session.createProducer(destination);

            TextMessage message;
            System.out.println(CLASS + ": Creating a TestMessage to send to the destination");

            for (int i = 1; i < numMessages + 1; i++)
            {
                message = session.createTextMessage("Message " + i);
                message.setIntProperty("seq",i);
                System.out.println(CLASS + ": Sending message: " + i);
                messageProducer.send(message, deliveryMode, Message.DEFAULT_PRIORITY, Message.DEFAULT_TIME_TO_LIVE);
            }

            message = session.createTextMessage("That's all, folks!");
            messageProducer.send(message, deliveryMode, Message.DEFAULT_PRIORITY, Message.DEFAULT_TIME_TO_LIVE);

            System.out.println(CLASS + ": Closing connection");
            connection.close();

            System.out.println(CLASS + ": Closing JNDI context");
            ctx.close();
        }
        catch (Exception exp)
        {
            System.err.println(CLASS + ": Caught an Exception: " + exp);
            exp.printStackTrace();
        }
    }
}

